#include <ctype.h>
#include "config.h"
#include "callbacks.h"
#include "callbacks.h"
#include "libpoc.h"
#include "libsupport.h"
#include "libmsgs.h"
#include "commands.h"

#include "q660_globals.h"
#include "unpascal.h"
#include "logging.h"

void createStationThread (void) {
  if (context == NULL) {
    tpar_create *pcret = &gConfig.par_create ;
    pcret->call_state = q660_state_callback ;
    pcret->call_messages = q660_msgs_callback ;
    pcret->mini_firchain = NULL ;
    pcret->file_owner = &gConfig.fowner ;
    pcret->call_minidata = q660_mini_callback ;
    pcret->call_lowlatency = q660_low_latency_callback ;
    pcret->call_secdata = q660_onesec_callback ;
    if (gConfig.write_archive)
      pcret->call_aminidata = q660_amini_callback ;
    else
      pcret->call_aminidata = NULL ;
    lib_create_context (&context, pcret) ;
    if (pcret->resp_err == LIBERR_NOERR) {
      cfgphase = CFG_IDLE ;
    } else {
      string s ;
      context = NULL ;
      logMessage ("Error creating station thread: %s\n", lib_get_errstr(pcret->resp_err, s)) ;
    }
  } else {
    lib_change_state (context, LIBSTATE_TERM, LIBERR_CLOSED) ;
  }
}

void startConnection (boolean ignoreLastData) {
  tpar_register *preg = &gConfig.par_register ;
  if (ignoreLastData) {
    preg->opt_start = OST_NEW ; /* Ignore last data time received */
  }
  if (context) {
    enum tliberr err ;
    string s ;
    if (preg->opt_dynamic_ip)
      preg->q660id_address[0] = 0 ;
    err = lib_register (context, preg) ;
    if (err == LIBERR_NOERR)
      cfgphase = CFG_RUN ;
    else
      logMessage ("Error Starting Connection: %s\n", lib_get_errstr(err, s)) ;
  }
}

void stopConnection(void) {
  if (poc_context) {
    logMessage ("POC Receiver Stopped\n") ;
    lib_poc_stop (poc_context) ;
  }
  if (context)
    lib_destroy_context (&context) ;
#ifdef X86_WIN32
   WSACleanup () ;
#endif
}

void get_command_line (pchar s)
{
#ifndef X86_WIN32
  integer i ;
#endif

#ifdef X86_WIN32
  gets(s) ;
#else
  fgets(s, 250, stdin) ;
  for (i = 0 ; i < strlen(s) ; i++)
    if (s[i] < 0x20)
        {
          s[i] = 0 ; /* terminate at first control character */
          break ;
        }
#endif
}

char get_command_char (void)
{
  string s ;

  while (TRUE)
  {
    get_command_line(line) ;
    sscanf (line, "%s", s) ;
    if (strlen(s) >= 1)
        return toupper(s[0]) ;
  }
}

void run_loop(void) {
  while (TRUE)
  {
    printf("cfgphase = %d\n", cfgphase) ;
    switch (cfgphase) {
      case CFG_IDLE :
        break ;
      case CFG_CREATE :
        break ;
      case CFG_REG :
        break ;
      case CFG_FILE :
        break ;
      case CFG_RUN :
        break ;
    }
    switch (get_command_char()) {
      case 'Q':
        stopConnection() ;
        return ;
    }
  }
}
